package org.hornetq.core.example;

import java.lang.management.ManagementFactory;

import javax.management.MBeanServer;
import javax.management.MBeanServerInvocationHandler;
import javax.management.ObjectName;

import org.hornetq.api.core.SimpleString;
import org.hornetq.api.core.TransportConfiguration;
import org.hornetq.api.core.client.ClientConsumer;
import org.hornetq.api.core.client.ClientMessage;
import org.hornetq.api.core.client.ClientProducer;
import org.hornetq.api.core.client.ClientSession;
import org.hornetq.api.core.client.ClientSessionFactory;
import org.hornetq.api.core.client.HornetQClient;
import org.hornetq.api.core.client.ServerLocator;
import org.hornetq.api.core.management.ObjectNameBuilder;
import org.hornetq.api.core.management.QueueControl;
import org.hornetq.core.remoting.impl.netty.NettyConnectorFactory;

public class CoreQueueCountersExample {

	private static final SimpleString QUEUE_NAME = new SimpleString("TestQueue");
	private static final String MSG_TEXT = "This is a test, only a test";
	private static final String PROP_KEY = "COUNT";
	
	private static class MessageCounterThread extends Thread {

		private QueueControl queueControl = null;
		
		public MessageCounterThread() throws Exception {
			// retrieve a connection to HornetQ's MBeanServer
			ObjectName on = ObjectNameBuilder.DEFAULT.getQueueObjectName(QUEUE_NAME, QUEUE_NAME);
			MBeanServer mbs = ManagementFactory.getPlatformMBeanServer();
			queueControl = (QueueControl)MBeanServerInvocationHandler.newProxyInstance(mbs, on, QueueControl.class, false);
		}

		@Override
		public void run() {
			try {
				while (!(this.getState().equals(State.TERMINATED))) {
					System.out.println(queueControl.getMessageCount() + " messages in the queue");
					long count = queueControl.countMessages(null);
					System.out.println(count + " messages counted in the queue");
					Thread.sleep(50);
				}
			} catch (Exception e) {
				System.out.println("Error getting counts: " + e.getMessage());
			}
		}
		
	}
	
	public static void main(final String[] args) throws Exception {

		ServerLocator locator = null;
		ClientSessionFactory csf = null;
		ClientSession session = null;
		MessageCounterThread thread = null;
		try {
			thread = new MessageCounterThread();
			thread.start();

			locator = HornetQClient.createServerLocatorWithoutHA(new TransportConfiguration(NettyConnectorFactory.class.getName()));
			csf = locator.createSessionFactory();

			// Step 2. Create a core session.
			session = csf.createSession(true, true);

			// Step 3. Create a core producer for queue.outgoingQueue.
			ClientProducer cp = session.createProducer(QUEUE_NAME);

			for (int i = 0; i < 100; i++) {
				ClientMessage cm = session.createMessage(org.hornetq.api.core.Message.TEXT_TYPE, false);
				cm.putIntProperty(PROP_KEY, i);
				cm.getBodyBuffer().writeString(MSG_TEXT);
				cp.send(cm);
				System.out.println("Sent message: " + i);				
				Thread.sleep(100);
			}

			// Step 4. Create a core consumer for queue.incomingQueue.
			ClientConsumer cc = session.createConsumer(QUEUE_NAME);
			session.start();
			ClientMessage msg = cc.receive(100);
			int msgCount = 0;
			int count = 0;
			do {
				if (null != msg) {
					msg.acknowledge();
					count = msg.getIntProperty(PROP_KEY);
					System.out.println("Received message: " + count);
					msgCount++;
					Thread.sleep(100);
					msg = cc.receive(100);
				}
			} while (null != msg);
			
			if (msgCount != 100)
				System.err.println("Not all messages received.");
			
			//Counts are off, lets see if there are more messages.
			ClientConsumer cc2 = session.createConsumer(QUEUE_NAME);
			ClientMessage msg2 = cc2.receive(100);
			if (null != msg2) {
				throw new RuntimeException("Error: recieved more messages than we put onto queue");
			}
			
		} catch (Exception e) {
			e.printStackTrace();
			System.exit(1);
		} finally {
			
			thread.interrupt();
			thread.join();
	         if(session != null)
	         {
	            session.close();
	         }
	         if(csf != null)
	         {
	            csf.close();
	         }

	         if (locator != null)
	         {
	            locator.close();
	         }
		}
	}
}
